
#include "cdr_iarchive.hpp"
#include <boost/archive/basic_binary_iprimitive.ipp>

namespace boost {
namespace archive {

using marshall::Big;
using marshall::Little;

template class basic_binary_iprimitive<cdr_iarchive<Big>, std::istream>;
template class ordered_iarchive_impl<cdr_iarchive<Big>, Big>;

template class basic_binary_iprimitive<cdr_iarchive<Little>, std::istream>;
template class ordered_iarchive_impl<cdr_iarchive<Little>, Little>;

typedef ordered_binary_iprimitive<cdr_iarchive<Big>, std::istream, Big> cdr_big_iprimitive;
typedef ordered_binary_iprimitive<cdr_iarchive<Little>, std::istream, Little> cdr_little_iprimitive;

using marshall::conversion_function16_t;
using marshall::conversion_function32_t;
using marshall::conversion_function64_t;

template<>
void ordered_load<1, Big>(cdr_big_iprimitive* primitive, void* address)
{
  primitive->load_binary(address, 1);
}
template<>
void ordered_load<1, Little>(cdr_little_iprimitive* primitive, void* address)
{
  primitive->load_binary(address, 1);
}

template<>
void ordered_load<2, Big>(cdr_big_iprimitive* primitive, void* address)
{
  do_ordered_load<uint16_t, conversion_function16_t, Big, cdr_big_iprimitive>(primitive, address);
}
template<>
void ordered_load<2, Little>(cdr_little_iprimitive* primitive, void* address)
{
  do_ordered_load<uint16_t, conversion_function16_t, Little, cdr_little_iprimitive>(primitive, address);
}

template<>
void ordered_load<4, Big>(cdr_big_iprimitive* primitive, void* address)
{
  do_ordered_load<uint32_t, conversion_function32_t, Big, cdr_big_iprimitive>(primitive, address);
}
template<>
void ordered_load<4, Little>(cdr_little_iprimitive* primitive, void* address)
{
  do_ordered_load<uint32_t, conversion_function32_t, Little, cdr_little_iprimitive>(primitive, address);
}

#ifndef BOOST_NO_INT64_T
template<>
void ordered_load<8, Big>(cdr_big_iprimitive* primitive, void* address)
{
  do_ordered_load<uint64_t, conversion_function64_t, Big, cdr_big_iprimitive>(primitive, address);
}
template<>
void ordered_load<8, Little>(cdr_little_iprimitive* primitive, void* address)
{
  do_ordered_load<uint64_t, conversion_function64_t, Little, cdr_little_iprimitive>(primitive, address);
}
#endif

} // namespace archive
} // namespace boost

