
#ifndef CDR_IARCHIVE_HPP
#define CDR_IARCHIVE_HPP

#include <iostream>

#include "ordered_iarchive.hpp"
#include "natural_alignment_distance.hpp"

namespace boost {
namespace archive {

using marshall::byte_ordering_t;

template<byte_ordering_t Order>
struct cdr_iarchive : 
    public ordered_iarchive_impl<cdr_iarchive<Order>, Order>
{
    cdr_iarchive(std::istream& is) :
        ordered_iarchive_impl<cdr_iarchive<Order>, Order>(is) {}

    // All elements are pre-aligned to their natural alignment
    template<typename T>
    void pre_align(const T&) 
    {
        // 'Natural' alignment
        size_t advance_bytes = natural_alignment_distance<sizeof(T)>(is.tellg());

        if (advance_bytes)
        {
            static char buffer[sizeof(T)];
            is.read(buffer, advance_bytes);
        }
    }

    // Default implementation same as for the ordered binary archive
    template<typename T>
    void load(T& t)
    {
        ordered_iarchive_impl<cdr_iarchive<Order>, Order>::load(t);
    }

    void load(std::string& s)
    {
        // The string is prepended with the 32-bit length including a terminator
        uint32_t length;
        load(length);

        s.resize(length - 1);
        ordered_iarchive_impl<cdr_iarchive<Order>, Order>::load(s);

        std::string::value_type terminator;
        load(terminator);
    }

    void load(std::wstring& ws)
    {
        // The string is prepended with the 32-bit length in bytes, not including a terminator
        uint32_t length;
        load(length);

        ws.resize(length / (sizeof(wchar_t) / sizeof(char)));
        ordered_iarchive_impl<cdr_iarchive<Order>, Order>::load(ws);
    }
};

} // namespace archive

namespace serialization {

// Provide default implementation for serializing vectors to network archives
template<marshall::byte_ordering_t Order, class T, class A>
inline void load(boost::archive::cdr_iarchive<Order> & ar, 
                 std::vector<T, A> & v,
                 const unsigned version)
{
    // Prepend the sequence with the number of elements
    uint32_t length;
    ar >> length;

    v.resize(length);
    for (typename std::vector<T, A>::iterator it = v.begin(); it != v.end(); ++it)
        ar >> (*it);
}

} // namespace serialization
} // namespace boost

#endif // CDR_IARCHIVE_HPP

