
#include "cdr_oarchive.hpp"
#include <boost/archive/basic_binary_oprimitive.ipp>

namespace boost {
namespace archive {

using marshall::Big;
using marshall::Little;

template class basic_binary_oprimitive<cdr_oarchive<Big>, std::ostream>;
template class ordered_oarchive_impl<cdr_oarchive<Big>, Big>;

template class basic_binary_oprimitive<cdr_oarchive<Little>, std::ostream>;
template class ordered_oarchive_impl<cdr_oarchive<Little>, Little>;

typedef ordered_binary_oprimitive<cdr_oarchive<Big>, std::ostream, Big> cdr_big_oprimitive;
typedef ordered_binary_oprimitive<cdr_oarchive<Little>, std::ostream, Little> cdr_little_oprimitive;

using marshall::conversion_function16_t;
using marshall::conversion_function32_t;
using marshall::conversion_function64_t;

template<>
void ordered_save<1, Big>(cdr_big_oprimitive* primitive, const void* address)
{
  primitive->save_binary(address, 1);
}
template<>
void ordered_save<1, Little>(cdr_little_oprimitive* primitive, const void* address)
{
  primitive->save_binary(address, 1);
}

template<>
void ordered_save<2, Big>(cdr_big_oprimitive* primitive, const void* address)
{
  do_ordered_save<uint16_t, conversion_function16_t, Big, cdr_big_oprimitive>(primitive, address);
}
template<>
void ordered_save<2, Little>(cdr_little_oprimitive* primitive, const void* address)
{
  do_ordered_save<uint16_t, conversion_function16_t, Little, cdr_little_oprimitive>(primitive, address);
}

template<>
void ordered_save<4, Big>(cdr_big_oprimitive* primitive, const void* address)
{
  do_ordered_save<uint32_t, conversion_function32_t, Big, cdr_big_oprimitive>(primitive, address);
}
template<>
void ordered_save<4, Little>(cdr_little_oprimitive* primitive, const void* address)
{
  do_ordered_save<uint32_t, conversion_function32_t, Little, cdr_little_oprimitive>(primitive, address);
}

#ifndef BOOST_NO_INT64_T
template<>
void ordered_save<8, Big>(cdr_big_oprimitive* primitive, const void* address)
{
  do_ordered_save<uint64_t, conversion_function64_t, Big, cdr_big_oprimitive>(primitive, address);
}
template<>
void ordered_save<8, Little>(cdr_little_oprimitive* primitive, const void* address)
{
  do_ordered_save<uint64_t, conversion_function64_t, Little, cdr_little_oprimitive>(primitive, address);
}
#endif

} // namespace archive
} // namespace boost

