
#ifndef ORDERED_TYPES_HPP
#define ORDERED_TYPES_HPP

#include <boost/cstdint.hpp>

#include <boost/mpl/list.hpp>
#include <boost/mpl/contains.hpp>
#include <boost/mpl/insert_range.hpp>

namespace boost {
namespace archive {

namespace ordered {

// Any natively 8-, 16-, 32- or 64-bit type can be serialized
typedef boost::mpl::list<float, double> real_types;

typedef boost::mpl::list<int8_t, uint8_t,
                         int16_t, uint16_t,
                         int32_t, uint32_t,
                         int64_t, uint64_t> integral_types;

// wchar_t's size is implementation-defined; it should probably be excluded...
typedef boost::mpl::list<char, wchar_t> char_types;

// Bool is also serializable, but must be overriden to enforce serialized size (8-bit)

// Strings and wstrings can also be serialized, but not directly

} // namespace ordered

// Define the types that can be serialized in an ordered archive
typedef boost::mpl::insert_range<
            ordered::integral_types, 
            boost::mpl::end<ordered::integral_types>::type,
            boost::mpl::insert_range<
                ordered::real_types,
                boost::mpl::end<ordered::real_types>::type,
                ordered::char_types
            >::type
        >::type ordered_types;

} // namespace archive
} // namespace boost

#endif

