
#include <iostream>
#include <sstream>
#include <fstream>
#include <cassert>

#include "test.hpp"

#include "ordered_oarchive.hpp"
#include "ordered_iarchive.hpp"
#include "xdr_oarchive.hpp"
#include "xdr_iarchive.hpp"
#include "cdr_oarchive.hpp"
#include "cdr_iarchive.hpp"

template<typename TestType, typename OutputArchiveType, typename InputArchiveType>
void do_test(const std::string& name)
{
    // Create initialized test object
    TestType t1;

    // Serialize object to binary format, in network byte order
    std::ostringstream oss;
    OutputArchiveType oa(oss);
    oa << t1;

    // Write the data to file
    std::string output_filename = "out." + name;
    std::ofstream ofs(output_filename.c_str());

    std::string output = oss.str();
    ofs.write(output.c_str(), output.size());
    ofs.close();
    
    // Create uninitialized object
    TestType t2(false);

    // De-serialize object from binary format, in network byte order
    std::istringstream iss(output);
    InputArchiveType ia(iss);
    ia >> t2;

    std::cout << name << " : Result of (t1 == t2): " << ((t1 == t2) ? "true" : "false") << std::endl;
    assert(t1 == t2);
}

int main(int, char**)
{
    // Simple binary ordered archive
    do_test<test_ordered, 
            boost::archive::ordered_oarchive<marshall::Big>,
            boost::archive::ordered_iarchive<marshall::Big> >("Ordered");

    // XDR archive 
    do_test<test_xdr, 
            boost::archive::xdr_oarchive,
            boost::archive::xdr_iarchive>("XDR");

    // CDR archive 
    do_test<test_cdr, 
            boost::archive::cdr_oarchive<marshall::Little>,
            boost::archive::cdr_iarchive<marshall::Little> >("CDR");
    
    return 0;
}

