#ifndef UNINDENTIFIED_ARCHIVE_HPP
#define UNINDENTIFIED_ARCHIVE_HPP

#include <boost/archive/basic_binary_iarchive.hpp>
#include <boost/archive/basic_binary_oarchive.hpp>

namespace boost { 
namespace archive {

// An 'unidentified' archive is one that whose type cannot be determined
// through inspection of its content (perhaps 'opaque'?)

// Base-class for an input archive containing no meta-data
template<class Archive>
class unidentified_iarchive : 
    public basic_binary_iarchive<Archive>
{
    // Don't load any meta-data 
    virtual void vload(version_type & t) {}
    virtual void vload(object_id_type & t) {}
    virtual void vload(class_id_type & t) {}
    virtual void vload(class_id_optional_type & t) {}
    virtual void vload(class_name_type & t) {}
    virtual void vload(tracking_type & t) {}
};

// Base-class for an output archive containing no meta-data
template<class Archive>
class unidentified_oarchive : 
    public basic_binary_oarchive<Archive>
{
    // Don't save any meta-data 
    virtual void vsave(const version_type t) {}
    virtual void vsave(const object_id_type t) {}
    virtual void vsave(const object_reference_type t) {}
    virtual void vsave(const class_id_type t) {}
    virtual void vsave(const class_id_reference_type t) {}
    virtual void vsave(const class_id_optional_type t) {}
    virtual void vsave(const class_name_type & t) {}
    virtual void vsave(const tracking_type t) {}
};

} // namespace archive
} // namespace boost

#endif // UNINDENTIFIED_ARCHIVE_HPP
