
#ifndef XDR_IARCHIVE_HPP
#define XDR_IARCHIVE_HPP

#include <iostream>

#include "ordered_iarchive.hpp"

#include "xdr_types.hpp"

namespace boost {
namespace archive {

using marshall::Big;

struct xdr_iarchive : 
    public ordered_iarchive_impl<xdr_iarchive, Big>
{
    xdr_iarchive(std::istream& is) :
        ordered_iarchive_impl<xdr_iarchive, Big>(is) {}

    // Default alignment - no post-alignment
    template<typename T>
    void post_align(const T&) {}

    // Align the next element after these elements to a 4-byte boundary
    void post_align(const std::string&) { do_post_align(is.tellg()); } 

    template<template<typename T> class A>
    void post_align(const std::vector<char, A<char> >&) { do_post_align(is.tellg()); } 
    template<template<typename T> class A>
    void post_align(const std::vector<unsigned char, A<unsigned char> >&) { do_post_align(is.tellg()); } 

    // Default implementation same as ordered binary archive
    template<typename T>
    void load(T& t,
              typename boost::enable_if<boost::mpl::contains<xdr_types, T>, T>::type* dummy = 0)
    {
        ordered_iarchive_impl<xdr_iarchive, Big>::load(t);
    }

    // Override bool to force 32-bit storage
    void load(bool& b)
    {
        int32_t value;
        load(value);

        b = (value ? true : false);
    }

    void load(std::string& s)
    {
        // The string is prepended with the 32-bit length
        uint32_t length;
        load(length);

        s.resize(length);
        ordered_iarchive_impl<xdr_iarchive, Big>::load(s);
    }

    void load(short int&)
    {
      // I can't understand why this is necessary...
    }

protected:
    void do_post_align(int position)
    {
      // Ensure that the next element is on a 4-byte boundary
      if (position % 4)
      {
        static char buffer[4] = { 0 };
        is.read(buffer, 4 - (position % 4));
      }
    }
};

} // namespace archive

namespace serialization {

// Provide default implementation for serializing vectors to network archives
template<class T, class A>
inline void load(boost::archive::xdr_iarchive & ar, 
                 std::vector<T, A> & v,
                 const unsigned version)
{
    // Prepend the sequence with the number of elements
    uint32_t length;
    ar >> length;

    v.resize(length);
    for (typename std::vector<T, A>::iterator it = v.begin(); it != v.end(); ++it)
        ar >> (*it);

    ar.post_align(v);
}

} // namespace serialization
} // namespace boost

#endif // XDR_IARCHIVE_HPP

